@@ -1,5 +1,5 @@
 /*
- *  $Id: gimp-print.h.in,v 1.8 2001/11/06 02:44:42 rlk Exp $
+ *  $Id: gimp-print.h.in,v 1.8.2.1 2002/12/08 21:12:33 rlk Exp $
  *
  *   Print plug-in header file for the GIMP.
  *
@@ -52,10 +52,10 @@ extern "C" {
  */
 #define GIMPPRINT_MAJOR_VERSION       (4)
 #define GIMPPRINT_MINOR_VERSION       (2)
-#define GIMPPRINT_MICRO_VERSION       (1)
-#define GIMPPRINT_CURRENT_INTERFACE   (1)
-#define GIMPPRINT_BINARY_AGE          (0)
-#define GIMPPRINT_INTERFACE_AGE       (1)
+#define GIMPPRINT_MICRO_VERSION       (5)
+#define GIMPPRINT_CURRENT_INTERFACE   (2)
+#define GIMPPRINT_BINARY_AGE          (1)
+#define GIMPPRINT_INTERFACE_AGE       (0)
 #define GIMPPRINT_CHECK_VERSION(major,minor,micro) \
   (GIMPPRINT_MAJOR_VERSION >  (major) || \
   (GIMPPRINT_MAJOR_VERSION == (major) && GIMPPRINT_MINOR_VERSION > (minor)) || \
@@ -106,6 +106,12 @@ typedef enum stp_papersize_unit
   PAPERSIZE_ENGLISH,
   PAPERSIZE_METRIC
 } stp_papersize_unit_t;
+  
+typedef enum
+{
+  STP_JOB_MODE_PAGE,
+  STP_JOB_MODE_JOB
+} stp_job_mode_t;
 
 typedef enum stp_image_status
 {
@@ -353,6 +359,10 @@ typedef struct
   void  (*describe_resolution)(const stp_printer_t printer,
                                const char *resolution, int *x, int *y);
   int   (*verify)(const stp_printer_t p, const stp_vars_t v);
+  int   (*start_job)(const stp_printer_t printer,
+		     stp_image_t *image, const stp_vars_t v);
+  int   (*end_job)(const stp_printer_t printer,
+		   stp_image_t *image, const stp_vars_t v);
 } stp_printfuncs_t;
 
 /*
@@ -538,6 +548,10 @@ extern unsigned stp_papersize_get_bottom (const stp_papersize_t pt);
 extern unsigned stp_papersize_get_right (const stp_papersize_t pt);
 extern stp_papersize_unit_t stp_papersize_get_unit (const stp_papersize_t pt);
 
+extern void stp_set_job_mode(stp_vars_t, stp_job_mode_t);
+extern stp_job_mode_t stp_get_job_mode(const stp_vars_t);
+extern void stp_set_page_number(stp_vars_t, int);
+extern int stp_get_page_number(const stp_vars_t);
 
 extern int stp_known_printers (void);
 extern const stp_printer_t stp_get_printer_by_index (int idx);
@@ -580,5 +594,5 @@ extern const stp_vars_t stp_minimum_settings (void);
 
 #endif /* __GIMP_PRINT_H__ */
 /*
- * End of $Id: gimp-print.h.in,v 1.8 2001/11/06 02:44:42 rlk Exp $
+ * End of $Id: gimp-print.h.in,v 1.8.2.1 2002/12/08 21:12:33 rlk Exp $
  */
