@@ -1,9 +1,9 @@
 /*
- * "$Id: genppd.c,v 1.36.2.3 2002/04/27 00:55:41 rlk Exp $"
+ * "$Id: genppd.c,v 1.36.2.8 2002/12/22 14:54:56 rlk Exp $"
  *
  *   PPD file generation program for the CUPS drivers.
  *
- *   Copyright 1993-2001 by Easy Software Products.
+ *   Copyright 1993-2002 by Easy Software Products.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License,
@@ -372,6 +372,11 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
   const stp_printfuncs_t *printfuncs;	/* Driver functions */
   paper_t	*the_papers;		/* Media sizes */
   int		cur_opt;		/* Current option */
+  int		variable_sizes;		/* Does the driver support variable sizes? */
+  int		min_width,		/* Min/max custom size */
+		min_height,
+		max_width,
+		max_height;
 
 
  /*
@@ -424,7 +429,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
     fprintf(stderr, ".");
 
   gzputs(fp, "*PPD-Adobe: \"4.3\"\n");
-  gzputs(fp, "*%PPD file for CUPS/GIMP-print.\n");
+  gzputs(fp, "*%PPD file for CUPS/Gimp-Print.\n");
   gzputs(fp, "*%Copyright 1993-2001 by Easy Software Products, All Rights Reserved.\n");
   gzputs(fp, "*%This PPD file may be freely used and distributed under the terms of\n");
   gzputs(fp, "*%the GNU GPL.\n");
@@ -440,11 +445,15 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
   gzprintf(fp, "*LanguageEncoding: %s\n", _("ISOLatin1"));
   gzprintf(fp, "*PCFileName:	\"%s.ppd\"\n", driver);
   gzprintf(fp, "*Manufacturer:	\"%s\"\n", manufacturer);
-  gzputs(fp, "*Product:	\"(GIMP-print v" VERSION ")\"\n");
+  gzputs(fp, "*Product:	\"(Gimp-Print v" VERSION ")\"\n");
   gzprintf(fp, "*ModelName:     \"%s\"\n", driver);
   gzprintf(fp, "*ShortNickName: \"%s\"\n", long_name);
-  gzprintf(fp, "*NickName:      \"%s, CUPS+GIMP-print v" VERSION "\"\n", long_name);
-  gzputs(fp, "*PSVersion:	\"(3010.000) 550\"\n");
+  gzprintf(fp, "*NickName:      \"%s, CUPS+Gimp-Print v" VERSION "\"\n", long_name);
+#if CUPS_PPD_PS_LEVEL == 2
+  gzputs(fp, "*PSVersion:	\"(2017.000) 705\"\n");
+#else
+  gzputs(fp, "*PSVersion:	\"(3010.000) 705\"\n");
+#endif /* CUPS_PPD_PS_LEVEL == 2 */
   gzprintf(fp, "*LanguageLevel:	\"%d\"\n", CUPS_PPD_PS_LEVEL);
   gzprintf(fp, "*ColorDevice:	%s\n",
            stp_get_output_type(printvars) == OUTPUT_COLOR ? "True" : "False");
@@ -460,13 +469,14 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
   gzputs(fp, "*cupsFilter:	\"application/vnd.cups-raster 100 rastertoprinter\"\n");
   if (strcasecmp(manufacturer, "EPSON") == 0)
     gzputs(fp, "*cupsFilter:	\"application/vnd.cups-command 33 commandtoepson\"\n");
+  gzputs(fp, "\n");
 
  /*
   * Get the page sizes from the driver...
   */
 
   v = stp_allocate_copy(printvars);
-
+  variable_sizes = 0;
   opts = (*(printfuncs->parameters))(p, NULL, "PageSize", &num_opts);
   defopt = (*(printfuncs->default_parameters))(p, NULL, "PageSize");
   the_papers = malloc(sizeof(paper_t) * num_opts);
@@ -481,6 +491,12 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
       continue;
     }
 
+    if (strcmp(opts[i].name, "Custom") == 0)
+    {
+      variable_sizes = 1;
+      continue;
+    }
+
     width  = stp_papersize_get_width(papersize);
     height = stp_papersize_get_height(papersize);
 
@@ -504,35 +520,31 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
     cur_opt++;
   }
 
+  gzprintf(fp, "*VariablePaperSize: %s\n\n", variable_sizes ? "true" : "false");
+
   gzputs(fp, "*OpenUI *PageSize: PickOne\n");
   gzputs(fp, "*OrderDependency: 10 AnySetup *PageSize\n");
-  gzputs(fp, "*DefaultPageSize: ");
-  gzputs(fp, defopt);
-  gzputs(fp, "\n");
+  gzprintf(fp, "*DefaultPageSize: %s\n", defopt);
   for (i = 0; i < cur_opt; i ++)
   {
     gzprintf(fp,  "*PageSize %s", the_papers[i].name);
     gzprintf(fp, "/%s:\t\"<</PageSize[%d %d]/ImagingBBox null>>setpagedevice\"\n",
              the_papers[i].text, the_papers[i].width, the_papers[i].height);
   }
-  gzputs(fp, "*CloseUI: *PageSize\n");
+  gzputs(fp, "*CloseUI: *PageSize\n\n");
 
   gzputs(fp, "*OpenUI *PageRegion: PickOne\n");
   gzputs(fp, "*OrderDependency: 10 AnySetup *PageRegion\n");
-  gzputs(fp, "*DefaultPageRegion: ");
-  gzputs(fp, defopt);
-  gzputs(fp, "\n");
+  gzprintf(fp, "*DefaultPageRegion: %s\n", defopt);
   for (i = 0; i < cur_opt; i ++)
   {
     gzprintf(fp,  "*PageRegion %s", the_papers[i].name);
-    gzprintf(fp, "/%s:\t\"<</PageRegion[%d %d]/ImagingBBox null>>setpagedevice\"\n",
+    gzprintf(fp, "/%s:\t\"<</PageSize[%d %d]/ImagingBBox null>>setpagedevice\"\n",
 	     the_papers[i].text, the_papers[i].width, the_papers[i].height);
   }
-  gzputs(fp, "*CloseUI: *PageRegion\n");
+  gzputs(fp, "*CloseUI: *PageRegion\n\n");
 
-  gzputs(fp, "*DefaultImageableArea: ");
-  gzputs(fp, defopt);
-  gzputs(fp, "\n");
+  gzprintf(fp, "*DefaultImageableArea: %s\n", defopt);
   for (i = 0; i < cur_opt; i ++)
   {
     gzprintf(fp,  "*ImageableArea %s", the_papers[i].name);
@@ -540,16 +552,39 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
              the_papers[i].left, the_papers[i].bottom,
 	     the_papers[i].right, the_papers[i].top);
   }
-
-  gzputs(fp, "*DefaultPaperDimension: ");
-  gzputs(fp, defopt);
   gzputs(fp, "\n");
+
+  gzprintf(fp, "*DefaultPaperDimension: %s\n", defopt);
+
   for (i = 0; i < cur_opt; i ++)
   {
     gzprintf(fp, "*PaperDimension %s", the_papers[i].name);
     gzprintf(fp, "/%s:\t\"%d %d\"\n",
 	     the_papers[i].text, the_papers[i].width, the_papers[i].height);
   }
+  gzputs(fp, "\n");
+
+  if (variable_sizes)
+  {
+    (*(printfuncs->limit))(p, v, &max_width, &max_height,
+			   &min_width, &min_height);
+    stp_set_media_size(v, "Custom");
+    (*(printfuncs->media_size))(p, v, &width, &height);
+    (*(printfuncs->imageable_area))(p, v, &left, &right, &bottom, &top);
+
+    gzprintf(fp, "*MaxMediaWidth:  \"%d\"\n", max_width);
+    gzprintf(fp, "*MaxMediaHeight: \"%d\"\n", max_height);
+    gzprintf(fp, "*HWMargins:      %d %d %d %d\n",
+	     left, bottom, width - right, height - top);
+    gzputs(fp, "*CustomPageSize True: \"pop pop pop <</PageSize[5 -2 roll]/ImagingBBox null>>setpagedevice\"\n");
+    gzprintf(fp, "*ParamCustomPageSize Width:        1 points %d %d\n",
+             min_width, max_width);
+    gzprintf(fp, "*ParamCustomPageSize Height:       2 points %d %d\n",
+             min_height, max_height);
+    gzputs(fp, "*ParamCustomPageSize WidthOffset:  3 points 0 0\n");
+    gzputs(fp, "*ParamCustomPageSize HeightOffset: 4 points 0 0\n");
+    gzputs(fp, "*ParamCustomPageSize Orientation:  5 int 0 0\n\n");
+  }
 
   if (opts)
   {
@@ -602,7 +637,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
              CUPS_CSPACE_CMYK, CUPS_ORDER_CHUNKED);
   }
 
-  gzputs(fp, "*CloseUI: *ColorModel\n");
+  gzputs(fp, "*CloseUI: *ColorModel\n\n");
 
  /*
   * Media types...
@@ -627,7 +662,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
 
     free(opts);
 
-    gzputs(fp, "*CloseUI: *MediaType\n");
+    gzputs(fp, "*CloseUI: *MediaType\n\n");
   }
 
  /*
@@ -653,7 +688,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
 
     free(opts);
 
-    gzputs(fp, "*CloseUI: *InputSlot\n");
+    gzputs(fp, "*CloseUI: *InputSlot\n\n");
   }
 
  /*
@@ -691,7 +726,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
 
   free(opts);
 
-  gzputs(fp, "*CloseUI: *Resolution\n");
+  gzputs(fp, "*CloseUI: *Resolution\n\n");
 
  /*
   * STP option group...
@@ -714,7 +749,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
     gzprintf(fp, "*stpImageType Continuous/%s:\t\"<</cupsRowCount 2>>setpagedevice\"\n",
             _("Photograph"));
 
-    gzputs(fp, "*CloseUI: *stpImageType\n");
+    gzputs(fp, "*CloseUI: *stpImageType\n\n");
 
    /*
     * Dithering algorithms...
@@ -728,7 +763,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
       gzprintf(fp, "*stpDither %s/%s: \"<</cupsRowStep %d>>setpagedevice\"\n",
                stp_dither_algorithm_name(i), stp_dither_algorithm_text(i), i);
 
-    gzputs(fp, "*CloseUI: *stpDither\n");
+    gzputs(fp, "*CloseUI: *stpDither\n\n");
 
    /*
     * InkTypes...
@@ -757,7 +792,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
 
       free(opts);
 
-      gzputs(fp, "*CloseUI: *stpInkType\n");
+      gzputs(fp, "*CloseUI: *stpInkType\n\n");
     }
 
    /*
@@ -778,14 +813,14 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
            j <= stp_options[i].high;
 	   j += stp_options[i].step)
 	gzprintf(fp, "*%s %d/%.3f: \"\"\n", stp_options[i].name, j, j * 0.001);
-      gzprintf(fp, "*CloseUI: *%s\n", stp_options[i].name);
+      gzprintf(fp, "*CloseUI: *%s\n\n", stp_options[i].name);
     }
 
  /*
   * End of STP option group...
   */
 
-  gzputs(fp, "*CloseGroup: STP\n");
+  gzputs(fp, "*CloseGroup: STP\n\n");
 
  /*
   * Fonts...
@@ -828,7 +863,7 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
   gzputs(fp, "*Font ZapfChancery-MediumItalic: Standard \"(001.007S)\" Standard ROM\n");
   gzputs(fp, "*Font ZapfDingbats: Special \"(001.004S)\" Standard ROM\n");
 
-  gzprintf(fp, "*%%End of %s.ppd\n", driver);
+  gzprintf(fp, "\n*%%End of %s.ppd\n", driver);
 
   gzclose(fp);
 
@@ -837,5 +872,5 @@ write_ppd(const stp_printer_t p,	/* I - Printer driver */
 }
 
 /*
- * End of "$Id: genppd.c,v 1.36.2.3 2002/04/27 00:55:41 rlk Exp $".
+ * End of "$Id: genppd.c,v 1.36.2.8 2002/12/22 14:54:56 rlk Exp $".
  */
