@@ -1,5 +1,5 @@
 /*
- * "$Id: escputil.c,v 1.39.2.3 2002/02/27 01:38:09 rlk Exp $"
+ * "$Id: escputil.c,v 1.39.2.10 2003/01/24 02:52:17 rlk Exp $"
  *
  *   Printer maintenance utility for EPSON Stylus (R) printers
  *
@@ -200,9 +200,16 @@ stp_printer_t printer_list[] =
   { "C20ux",	N_("Stylus C20ux"),	3,	15,	0,	2,	9 },
   { "C40sx",	N_("Stylus C40sx"),	3,	15,	0,	2,	9 },
   { "C40ux",	N_("Stylus C40ux"),	3,	15,	0,	2,	9 },
+  { "C41sx",	N_("Stylus C41sx"),	3,	15,	0,	2,	9 },
+  { "C41ux",	N_("Stylus C41ux"),	3,	15,	0,	2,	9 },
+  { "C42sx",	N_("Stylus C42sx"),	3,	15,	0,	2,	9 },
+  { "C42ux",	N_("Stylus C42ux"),	3,	15,	0,	2,	9 },
   { "C60",	N_("Stylus C60"),	3,	15,	0,	0,	0 },
+  { "C61",	N_("Stylus C61"),	3,	15,	0,	0,	0 },
+  { "C62",	N_("Stylus C62"),	3,	15,	0,	0,	0 },
   { "C70",	N_("Stylus C70"),	4,	15,	0,	1,	7 },
   { "C80",	N_("Stylus C80"),	4,	15,	0,	1,	7 },
+  { "C82",	N_("Stylus C82"),	4,	15,	0,	1,	7 },
   { "color",	N_("Stylus Color"),	1,	7,	0,	0,	0 },
   { "pro",	N_("Stylus Color Pro"),	1,	7,	0,	0,	0 },
   { "pro-xl",	N_("Stylus Color Pro XL"),1,	7,	0,	0,	0 },
@@ -241,21 +248,30 @@ stp_printer_t printer_list[] =
   { "790",	N_("Stylus Photo 790"),	3,	15,	0,	0,	0 },
   { "810",	N_("Stylus Photo 810"),	3,	15,	0,	0,	0 },
   { "820",	N_("Stylus Photo 820"),	3,	15,	0,	0,	0 },
+  { "830",	N_("Stylus Photo 830"),	3,	15,	0,	0,	0 },
   { "870",	N_("Stylus Photo 870"),	3,	15,	0,	0,	0 },
   { "875",	N_("Stylus Photo 875"),	3,	15,	0,	0,	0 },
   { "890",	N_("Stylus Photo 890"),	3,	15,	0,	0,	0 },
   { "895",	N_("Stylus Photo 895"),	3,	15,	0,	0,	0 },
+  { "915",	N_("Stylus Photo 915"),	3,	15,	0,	0,	0 },
+  { "925",	N_("Stylus Photo 925"),	3,	15,	0,	0,	0 },
+  { "950",	N_("Stylus Photo 950"),	4,	15,	0,	0,	0 },
+  { "960",	N_("Stylus Photo 960"),	4,	15,	0,	0,	0 },
   { "1200",	N_("Stylus Photo 1200"),3,	15,	0,	0,	0 },
   { "1270",	N_("Stylus Photo 1270"),3,	15,	0,	0,	0 },
   { "1280",	N_("Stylus Photo 1280"),3,	15,	0,	0,	0 },
   { "1290",	N_("Stylus Photo 1290"),3,	15,	0,	0,	0 },
   { "2000",	N_("Stylus Photo 2000P"),2,	15,	0,	0,	0 },
+  { "2100",	N_("Stylus Photo 2100"),4,	15,	0,	0,	0 },
+  { "2200",	N_("Stylus Photo 2200"),4,	15,	0,	0,	0 },
   { "5000",	N_("Stylus Pro 5000"),	1,	7,	0,	0,	0 },
   { "5500",	N_("Stylus Pro 5500"),	1,	7,	0,	0,	0 },
   { "7000",	N_("Stylus Pro 7000"),	1,	7,	0,	0,	0 },
   { "7500",	N_("Stylus Pro 7500"),	1,	7,	0,	0,	0 },
+  { "7600",	N_("Stylus Pro 7600"),	3,	15,	0,	0,	0 },
   { "9000",	N_("Stylus Pro 9000"),	1,	7,	0,	0,	0 },
   { "9500",	N_("Stylus Pro 9500"),	1,	7,	0,	0,	0 },
+  { "9600",	N_("Stylus Pro 9600"),	3,	15,	0,	0,	0 },
   { "10000",	N_("Stylus Pro 10000"),	3,	15,	0,	0,	0 },
   { "scan2000",	N_("Stylus Scan 2000"),	3,	15,	0,	0,	0 },
   { "scan2500",	N_("Stylus Scan 2500"),	3,	15,	0,	0,	0 },
@@ -457,12 +473,12 @@ do_print_cmd(void)
         if (printer == NULL)
           strcpy(command, "lpr -l");
 	else
-          sprintf(command, "lpr -P%s -l", printer);
+          snprintf(command, 1023, "lpr -P%s -l", printer);
         }
       else if (printer == NULL)
 	strcpy(command, "lp -s -oraw");
       else
-	sprintf(command, "lp -s -oraw -d%s", printer);
+	snprintf(command, 1023, "lp -s -oraw -d%s", printer);
 
       if ((pfile = popen(command, "w")) == NULL)
 	{
@@ -587,6 +603,7 @@ const char *colors[] =
   N_("Yellow"),
   N_("Light Cyan"),
   N_("Light Magenta"),
+  N_("Black/Dark Yellow"),
   0
 };
 
@@ -637,7 +654,7 @@ do_ink_level(void)
     }
   ind += 3;
   printf("%20s    %s\n", _("Ink color"), _("Percent remaining"));
-  for (i = 0; i < 6; i++)
+  for (i = 0; i < 7; i++)
     {
       int val, j;
       if (!ind[0] || ind[0] == ';')
