@@ -1,5 +1,5 @@
 /*
- *  $Id: ijsgimpprint.c,v 1.2.2.7 2002/04/04 12:16:39 rlk Exp $
+ *  $Id: ijsgimpprint.c,v 1.2.2.15 2002/12/08 21:12:40 rlk Exp $
  *
  *   ijs server for gimp-print.
  *
@@ -36,28 +36,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <locale.h>
 #include <ijs.h>
 #include <ijs_server.h>
 #include <errno.h>
 #include <gimp-print/gimp-print-intl-internal.h>
 
-/* WARNING:
- * gimp-print wants to pull the raster data.  
- * We can't push the raster data to gimp-print.  
- * All gimp-print drivers as of 2001-11-02 read rows on increasing 
- * order, sometimes with skipped rows.  They never ask for an earlier 
- * row.  To avoid keeping the entire image in memory we only buffer 
- * one row at a time and hope that the gimp-print drivers don't change 
- * their behaviour.
- * An interface like gdk_draw_rgb_image() would have been easier.
- */
-
 static int stp_debug = 0;
 
 #define STP_DEBUG(x) do { if (stp_debug || getenv("STP_DEBUG")) x; } while (0)
 
-#define BUF_SIZE 4096
-
 typedef struct _GimpParamList GimpParamList;
 
 struct _GimpParamList {
@@ -83,8 +71,8 @@ typedef struct _IMAGE
   int row;		/* row number in buffer */
   int row_width;	/* length of a row */
   char *row_buf;	/* buffer for raster */
-  int total_bytes;	/* total size of raster */
-  int bytes_left;	/* bytes remaining to be read */
+  double total_bytes;	/* total size of raster */
+  double bytes_left;	/* bytes remaining to be read */
   GimpParamList *params;
 } IMAGE;
 
@@ -92,6 +80,14 @@ static const char DeviceGray[] = "DeviceGray";
 static const char DeviceRGB[] = "DeviceRGB";
 static const char DeviceCMYK[] = "DeviceCMYK";
 
+static char *
+c_strdup(const char *s)
+{
+  char *ret = malloc(strlen(s) + 1);
+  strcpy(ret, s);
+  return ret;
+}
+
 static int
 image_init(IMAGE *img, IjsPageHeader *ph)
 {
@@ -102,7 +98,7 @@ image_init(IMAGE *img, IjsPageHeader *ph)
   img->xres = ph->xres;
   img->yres = ph->yres;
 
-  img->row = 0;
+  img->row = -1;
   img->row_width = (ph->n_chan * ph->bps * ph->width + 7) >> 3;
   img->row_buf = (char *)malloc(img->row_width);
   STP_DEBUG(fprintf(stderr, "image_init\n"));
@@ -169,8 +165,12 @@ get_float(const char *str, const char *name, float *pval,
 	  float min_value, float max_value)
 {
   float new_value;
+  /* Force locale to "C", because decimal numbers coming from the IJS
+     client are always with a decimal point, nver with a decimal comma */
+  setlocale(LC_ALL, "C");
   if (sscanf(str, "%f", &new_value) == 1)
     {
+      setlocale(LC_ALL, "");
       if ((new_value >= min_value) && (new_value <= max_value))
 	{
 	  *pval = new_value;
@@ -182,8 +182,11 @@ get_float(const char *str, const char *name, float *pval,
 		name, new_value, min_value, max_value);
     }
   else
-    fprintf(stderr, _("Unable to parse parameter %s=%s (expect a number)\n"),
-	    name, str);
+    {
+      setlocale(LC_ALL, "");
+      fprintf(stderr, _("Unable to parse parameter %s=%s (expect a number)\n"),
+	      name, str);
+    }
   return -1;
 }
 
@@ -192,8 +195,12 @@ get_int(const char *str, const char *name, int *pval,
 	int min_value, int max_value)
 {
   int new_value;
+  /* Force locale to "C", because decimal numbers sent to the IJS
+     client must have a decimal point, nver a decimal comma */
+  setlocale(LC_ALL, "C");
   if (sscanf(str, "%d", &new_value) == 1)
     {
+      setlocale(LC_ALL, "");
       if ((new_value >= min_value) && (new_value <= max_value))
 	{
 	  *pval = new_value;
@@ -204,8 +211,11 @@ get_int(const char *str, const char *name, int *pval,
 		name, new_value, min_value, max_value);
     }
   else
-    fprintf(stderr, _("Unable to parse parameter %s=%s (expect a number)\n"),
-	    name, str);
+    {
+      setlocale(LC_ALL, "");
+      fprintf(stderr, _("Unable to parse parameter %s=%s (expect a number)\n"),
+	      name, str);
+    }
   return -1;
 }
 
@@ -230,7 +240,11 @@ gimp_parse_wxh (const char *val, int size,
 
   memcpy (buf, val, i);
   buf[i] = 0;
+  /* Force locale to "C", because decimal numbers coming from the IJS
+     client are always with a decimal point, nver with a decimal comma */
+  setlocale(LC_ALL, "C");
   *pw = strtod (buf, &tail);
+  setlocale(LC_ALL, "");
   if (tail == buf)
     return IJS_ESYNTAX;
 
@@ -239,7 +253,11 @@ gimp_parse_wxh (const char *val, int size,
 
   memcpy (buf, val + i + 1, size - i - 1);
   buf[size - i - 1] = 0;
+  /* Force locale to "C", because decimal numbers coming from the IJS
+     client are always with a decimal point, nver with a decimal comma */
+  setlocale(LC_ALL, "C");
   *ph = strtod (buf, &tail);
+  setlocale(LC_ALL, "");
   if (tail == buf)
     return IJS_ESYNTAX;
 
@@ -365,7 +383,11 @@ gimp_get_cb (void *get_cb_data,
 	(printer, v, &l, &r, &b, &t);
       h = t - b;
       w = r - l;
+      /* Force locale to "C", because decimal numbers sent to the IJS
+	 client must have a decimal point, nver a decimal comma */
+      setlocale(LC_ALL, "C");
       sprintf(buf, "%gx%g", (double) w / 72.0, (double) h / 72.0);
+      setlocale(LC_ALL, "");
       STP_DEBUG(fprintf(stderr, "PrintableArea %d %d %s\n", h, w, buf));
       val = buf;
     }
@@ -374,7 +396,11 @@ gimp_get_cb (void *get_cb_data,
       int x, y;
       (*stp_printer_get_printfuncs(printer)->describe_resolution)
 	(printer, stp_get_resolution(v), &x, &y);
+      /* Force locale to "C", because decimal numbers sent to the IJS
+	 client must have a decimal point, nver a decimal comma */
+      setlocale(LC_ALL, "C");
       sprintf(buf, "%d", x);
+      setlocale(LC_ALL, "");
       STP_DEBUG(fprintf(stderr, "Dpi %d %d (%d) %s\n", x, y, x, buf));
       stp_set_scaling(v, -x);
       val = buf;
@@ -388,7 +414,11 @@ gimp_get_cb (void *get_cb_data,
       (*stp_printer_get_printfuncs(printer)->imageable_area)
 	(printer, v, &l, &r, &b, &t);
       t = h - t;
+      /* Force locale to "C", because decimal numbers sent to the IJS
+	 client must have a decimal point, nver a decimal comma */
+      setlocale(LC_ALL, "C");
       sprintf(buf, "%gx%g", (double) l / 72.0, (double) t / 72.0);
+      setlocale(LC_ALL, "");
       STP_DEBUG(fprintf(stderr, "PrintableTopLeft %d %d %s\n", t, l, buf));
       val = buf;
     }
@@ -435,11 +465,15 @@ gimp_set_cb (void *set_cb_data, IjsServerCtx *ctx, IjsJobId jobid,
     {
       if (img->filename)
 	free(img->filename);
-      img->filename = strdup(vbuf);
+      img->filename = c_strdup(vbuf);
     }
-  else if (strcmp(key, "OutputFD") == 0)
+  else if (strcmp(key, "OutputFD") == 0) {
+    /* Force locale to "C", because decimal numbers sent to the IJS
+       client must have a decimal point, nver a decimal comma */
+    setlocale(LC_ALL, "C");
     img->fd = atoi(vbuf) + 1;
-  else if (strcmp(key, "DeviceManufacturer") == 0)
+    setlocale(LC_ALL, "");
+  } else if (strcmp(key, "DeviceManufacturer") == 0)
     ;				/* We don't care who makes it */
   else if (strcmp(key, "DeviceModel") == 0)
     {
@@ -584,6 +618,18 @@ gimp_set_cb (void *set_cb_data, IjsServerCtx *ctx, IjsJobId jobid,
       if (code == 0)
 	stp_set_density(img->v, z);
     }
+  else if (strcmp (key, "Duplex") == 0)
+    {
+    }
+  else if (strcmp (key, "PS:Duplex") == 0)
+    {
+    }
+  else if (strcmp (key, "Tumble") == 0)
+    {
+    }
+  else if (strcmp (key, "PS:Tumble") == 0)
+    {
+    }
   else
     {
       fprintf(stderr, _("Unknown option %s\n"), key);
@@ -664,17 +710,17 @@ static int
 image_next_row(IMAGE *img)
 {
   int status = 0;
-  int n_bytes = img->bytes_left;
+  double n_bytes = img->bytes_left;
   if (img->bytes_left)
     {
 
       if (n_bytes > img->row_width)
 	n_bytes = img->row_width;
 #ifdef VERBOSE
-      STP_DEBUG(fprintf(stderr, "%d bytes left, reading %d, on row %d\n", 
-			bytes_left, n_bytes, img->row));
+      STP_DEBUG(fprintf(stderr, "%.0f bytes left, reading %.d, on row %d\n",
+			img->bytes_left, (int) n_bytes, img->row));
 #endif
-      status = ijs_server_get_data(img->ctx, img->row_buf, n_bytes);
+      status = ijs_server_get_data(img->ctx, img->row_buf, (int) n_bytes);
       if (status)
 	{
 	  STP_DEBUG(fprintf(stderr, "page aborted!\n"));
@@ -707,7 +753,38 @@ gimp_image_get_row(stp_image_t *image, unsigned char *data, int row)
     }
 
   if (physical_row == img->row)
-    memcpy(data, img->row_buf, img->row_width);
+    {
+      unsigned i, j, length;
+      switch (img->bps)
+	{
+	case 8:
+	  memcpy(data, img->row_buf, img->row_width);
+	  break;
+	case 1:
+	  length = img->width / 8;
+	  for (i = 0; i < length; i++)
+	    for (j = 128; j > 0; j >>= 1)
+	      {
+		if (img->row_buf[i] & j)
+		  data[0] = 255;
+		else
+		  data[0] = 0;
+		data++;
+	      }
+	  length = img->width % 8;
+	  for (j = 128; j > 1 << (7 - length); j >>= 1)
+	    {
+	      if (img->row_buf[i] & j)
+		data[0] = 255;
+	      else
+		data[0] = 0;
+	      data++;
+	    }
+	  break;
+	default:
+	  return STP_IMAGE_ABORT;
+	}
+    }
   else
     return STP_IMAGE_ABORT;
   return STP_IMAGE_OK;
@@ -833,7 +910,12 @@ main (int argc, char **argv)
 
       status = ijs_server_get_page_header(img.ctx, &ph);
       if (status)
-	break;
+	{
+	  if (status < 0)
+	    fprintf(stderr, _("ijs_server_get_page_header failed %d\n"),
+		    status);
+	  break;
+	}
       STP_DEBUG(fprintf(stderr, "got page header, %d x %d\n",
 			ph.width, ph.height));
       STP_DEBUG(stp_dbg("have page header", img.v));
@@ -890,19 +972,21 @@ main (int argc, char **argv)
 	    stp_set_dither_algorithm(img.v, stp_default_dither_algorithm());
 	}
 
-      page++;
-
-      img.total_bytes = ((ph.n_chan * ph.bps * ph.width + 7) >> 3) 
-	* ph.height;
+      img.total_bytes = (double) ((ph.n_chan * ph.bps * ph.width + 7) >> 3) 
+	* (double) ph.height;
       img.bytes_left = img.total_bytes;
 
       stp_set_app_gamma(img.v, (float)1.7);
       stp_set_cmap(img.v, NULL);
       stp_set_scaling(img.v, (float)-img.xres); /* resolution of image */
       stp_set_output_type(img.v, img.output_type); 
+      stp_set_page_number(img.v, page);
+      stp_set_job_mode(img.v, STP_JOB_MODE_JOB);
       STP_DEBUG(stp_dbg("about to print", img.v));
       if (stp_printer_get_printfuncs(printer)->verify(printer, img.v))
 	{
+	  if (page == 0)
+	    stp_printer_get_printfuncs(printer)->start_job(printer, &si, img.v);
 	  stp_printer_get_printfuncs(printer)->print(printer, &si, img.v);
 	}
       else
@@ -917,15 +1001,18 @@ main (int argc, char **argv)
 	  status = image_next_row(&img);
 	  if (status)
 	    {
-	      fprintf(stderr, _("Get next row failed at %d\n"),
+	      fprintf(stderr, _("Get next row failed at %.0f\n"),
 		      img.bytes_left);
 	      break;
 	    }
 	}
 
       image_finish(&img);
+      page++;
     }
   while (status == 0);
+  stp_printer_get_printfuncs(printer)->end_job(printer, &si, img.v);
+
   if (f)
     {
       fclose(f);
