@@ -1,5 +1,5 @@
 /*
- * "$Id: print-canon.c,v 1.71.2.1 2002/03/02 00:32:49 rlk Exp $"
+ * "$Id: print-canon.c,v 1.71.2.6 2002/12/08 21:12:42 rlk Exp $"
  *
  *   Print plug-in CANON BJL driver for the GIMP.
  *
@@ -216,8 +216,8 @@ typedef struct canon_variable_printmode
  */
 static const stp_simple_dither_range_t canon_dither_ranges_Cc_1bit[] =
 {
-  { 0.25, 0x1, 0, 1 },
-  { 1.0,  0x1, 1, 1 }
+  { 0.25, 0x1, 1, 1 },
+  { 1.0,  0x1, 0, 1 }
 };
 
 static const canon_variable_ink_t canon_ink_Cc_1bit =
@@ -233,8 +233,8 @@ static const canon_variable_ink_t canon_ink_Cc_1bit =
  */
 static const stp_simple_dither_range_t canon_dither_ranges_Mm_1bit[] =
 {
-  { 0.26, 0x1, 0, 1 },
-  { 1.0,  0x1, 1, 1 }
+  { 0.26, 0x1, 1, 1 },
+  { 1.0,  0x1, 0, 1 }
 };
 
 static const canon_variable_ink_t canon_ink_Mm_1bit =
@@ -251,9 +251,9 @@ static const canon_variable_ink_t canon_ink_Mm_1bit =
  */
 static const stp_simple_dither_range_t canon_dither_ranges_X_2bit[] =
 {
-  { 0.45,  0x1, 1, 1 },
-  { 0.68,  0x2, 1, 2 },
-  { 1.0,   0x3, 1, 3 }
+  { 0.45,  0x1, 0, 1 },
+  { 0.68,  0x2, 0, 2 },
+  { 1.0,   0x3, 0, 3 }
 };
 
 static const canon_variable_ink_t canon_ink_X_2bit =
@@ -269,12 +269,12 @@ static const canon_variable_ink_t canon_ink_X_2bit =
  */
 static const stp_simple_dither_range_t canon_dither_ranges_Xx_2bit[] =
 {
-  { 0.15,  0x1, 0, 1 },
-  { 0.227, 0x2, 0, 2 },
-/*  { 0.333, 0x3, 0, 3 }, */
-  { 0.45,  0x1, 1, 1 },
-  { 0.68,  0x2, 1, 2 },
-  { 1.0,   0x3, 1, 3 }
+  { 0.15,  0x1, 1, 1 },
+  { 0.227, 0x2, 1, 2 },
+/*{ 0.333, 0x3, 1, 3 }, */
+  { 0.45,  0x1, 0, 1 },
+  { 0.68,  0x2, 0, 2 },
+  { 1.0,   0x3, 0, 3 }
 };
 
 static const canon_variable_ink_t canon_ink_Xx_2bit =
@@ -297,12 +297,12 @@ static const canon_variable_ink_t canon_ink_Xx_2bit =
  */
 static const stp_simple_dither_range_t canon_dither_ranges_X_3bit[] =
 {
-  { 0.45,  0x1, 1, 1 },
-  { 0.55,  0x2, 1, 2 },
-  { 0.66,  0x3, 1, 3 },
-  { 0.77,  0x4, 1, 4 },
-  { 0.88,  0x5, 1, 5 },
-  { 1.0,   0x6, 1, 6 }
+  { 0.45,  0x1, 0, 1 },
+  { 0.55,  0x2, 0, 2 },
+  { 0.66,  0x3, 0, 3 },
+  { 0.77,  0x4, 0, 4 },
+  { 0.88,  0x5, 0, 5 },
+  { 1.0,   0x6, 0, 6 }
 };
 
 static const canon_variable_ink_t canon_ink_X_3bit =
@@ -318,16 +318,16 @@ static const canon_variable_ink_t canon_ink_X_3bit =
  */
 static const stp_simple_dither_range_t canon_dither_ranges_Xx_3bit[] =
 {
-  { 0.15,  0x1, 0, 1 },
-  { 0.227, 0x2, 0, 2 },
-  { 0.333, 0x3, 0, 3 },
-/*  { 0.333, 0x3, 0, 3 }, */
-  { 0.45,  0x1, 1, 1 },
-  { 0.55,  0x2, 1, 2 },
-  { 0.66,  0x3, 1, 3 },
-  { 0.77,  0x4, 1, 4 },
-  { 0.88,  0x5, 1, 5 },
-  { 1.0,   0x6, 1, 6 }
+  { 0.15,  0x1, 1, 1 },
+  { 0.227, 0x2, 1, 2 },
+  { 0.333, 0x3, 1, 3 },
+/*  { 0.333, 0x3, 1, 3 }, */
+  { 0.45,  0x1, 0, 1 },
+  { 0.55,  0x2, 0, 2 },
+  { 0.66,  0x3, 0, 3 },
+  { 0.77,  0x4, 0, 4 },
+  { 0.88,  0x5, 0, 5 },
+  { 1.0,   0x6, 0, 6 }
 };
 
 static const canon_variable_ink_t canon_ink_Xx_3bit =
@@ -936,7 +936,7 @@ static const canon_cap_t canon_model_capabilities[] =
 
   { /* Canon BJC 6500 *//* heads: BC-30/BC-31 BC-32/BC-31 */
     6500, 3,
-    11*72, 17*72,
+    842, 17*72,
     180, 1440, 720, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK | CANON_INK_CcMmYK,
@@ -955,7 +955,7 @@ static const canon_cap_t canon_model_capabilities[] =
 
   { /* Canon BJC 8200 *//* heads: BC-50 */
     8200, 3,
-    11*72, 17*72,
+    842, 17*72,
     150, 1200,1200, 4,
     11, 9, 10, 18,
     CANON_INK_CMYK, /*  | CANON_INK_CcMmYK */
@@ -982,7 +982,7 @@ static const canon_cap_t canon_model_capabilities[] =
 
   { /* Canon BJC 210 *//* heads: BC-02 BC-05 BC-06 */
     210, 1,
-    11*72, 17*72,
+    618, 936,      /* 8.58" x 13 " */
     90, 720, 360, 2,
     11, 9, 10, 18,
     CANON_INK_K | CANON_INK_CMY,
@@ -1000,7 +1000,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 240 *//* heads: BC-02 BC-05 BC-06 */
     240, 1,
-    11*72, 17*72,
+    618, 936,      /* 8.58" x 13 " */
     90, 720, 360, 2,
     11, 9, 10, 18,
     CANON_INK_K | CANON_INK_CMY,
@@ -1018,7 +1018,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 250 *//* heads: BC-02 BC-05 BC-06 */
     250, 1,
-    11*72, 17*72,
+    618, 936,      /* 8.58" x 13 " */
     90, 720, 360, 2,
     11, 9, 10, 18,
     CANON_INK_K | CANON_INK_CMY,
@@ -1036,7 +1036,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 1000 *//* heads: BC-02 BC-05 BC-06 */
     1000, 1,
-    11*72, 17*72,
+    842, 17*72,
     90, 720, 360, 2,
     11, 9, 10, 18,
     CANON_INK_K | CANON_INK_CMY,
@@ -1054,7 +1054,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 2000 *//* heads: BC-20 BC-21 BC-22 BC-29 */
     2000, 1,
-    11*72, 17*72,
+    842, 17*72,
     180, 720, 360, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK,
@@ -1072,7 +1072,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 3000 *//* heads: BC-30 BC-33 BC-34 */
     3000, 3,
-    11*72, 17*72,
+    842, 17*72,
     180, 1440, 720, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK | CANON_INK_CcMmYK,
@@ -1090,7 +1090,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 6100 *//* heads: BC-30/BC-31 BC-32/BC-31 */
     6100, 3,
-    11*72, 17*72,
+    842, 17*72,
     180, 1440, 720, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK | CANON_INK_CcMmYK,
@@ -1108,7 +1108,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 7000 *//* heads: BC-60/BC-61 BC-60/BC-62   ??????? */
     7000, 3,
-    11*72, 17*72,
+    842, 17*72,
     150, 1200, 600, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK | CANON_INK_CcMmYyK,
@@ -1126,7 +1126,7 @@ static const canon_cap_t canon_model_capabilities[] =
   },
   { /* Canon BJC 7100 *//* heads: BC-60/BC-61 BC-60/BC-62   ??????? */
     7100, 3,
-    11*72, 17*72,
+    842, 17*72,
     150, 1200, 600, 2,
     11, 9, 10, 18,
     CANON_INK_CMYK | CANON_INK_CcMmYyK,
@@ -1589,7 +1589,7 @@ canon_describe_resolution(const stp_printer_t printer,
   return;
 }
 
-static stp_param_t media_sources[] =
+static const stp_param_t media_sources[] =
               {
                 { "Auto",	N_ ("Auto Sheet Feeder") },
                 { "Manual",	N_ ("Manual with Pause") },
@@ -1608,7 +1608,6 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
                  int  *count)			/* O - Number of values */
 {
   int		i;
-  stp_param_t *p= 0;
   stp_param_t *valptrs= 0;
 
   const canon_cap_t * caps=
@@ -1626,7 +1625,7 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
   {
     int height_limit, width_limit;
     int papersizes = stp_known_papersizes();
-    valptrs = stp_malloc(sizeof(stp_param_t) * papersizes);
+    valptrs = stp_zalloc(sizeof(stp_param_t) * papersizes);
     *count = 0;
 
     width_limit = caps->max_width;
@@ -1650,7 +1649,7 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
     int x,y;
     int c= 0;
     int t;
-    valptrs = stp_malloc(sizeof(stp_param_t) * 10);
+    valptrs = stp_zalloc(sizeof(stp_param_t) * 10);
 
     for (x=1; x<6; x++) {
       for (y=x-1; y<x+1; y++) {
@@ -1681,7 +1680,7 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
   else if (strcmp(name, "InkType") == 0)
   {
     int c= 0;
-    valptrs = stp_malloc(sizeof(stp_param_t) * 5);
+    valptrs = stp_zalloc(sizeof(stp_param_t) * 5);
     /* used internally: do not translate */
     if ((caps->inks & CANON_INK_K))
     {
@@ -1715,7 +1714,7 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
   {
     *count = sizeof(canon_paper_list) / sizeof(canon_paper_list[0]);
 
-    valptrs = stp_malloc(*count * sizeof(stp_param_t));
+    valptrs = stp_zalloc(*count * sizeof(stp_param_t));
 
     for (i = 0; i < *count; i ++)
     {
@@ -1725,15 +1724,14 @@ canon_parameters(const stp_printer_t printer,	/* I - Printer model */
   }
   else if (strcmp(name, "InputSlot") == 0)
   {
-    *count = 3;
-    p = media_sources;
+    *count = sizeof(media_sources) / sizeof(media_sources[0]);
 
-    valptrs = stp_malloc(*count * sizeof(stp_param_t));
+    valptrs = stp_zalloc(*count * sizeof(stp_param_t));
     for (i = 0; i < *count; i ++)
     {
       /* translate media_sources */
-      valptrs[i].name = c_strdup(p[i].name);
-      valptrs[i].text = c_strdup(_(p[i].text));
+      valptrs[i].name = c_strdup(media_sources[i].name);
+      valptrs[i].text = c_strdup(_(media_sources[i].text));
     }
   }
   else
@@ -1878,7 +1876,7 @@ canon_cmd(const stp_vars_t v, /* I - the printer         */
 	  ...        /* I - the args themselves */
 	  )
 {
-  unsigned char *buffer = stp_malloc(num + 1);
+  unsigned char *buffer = stp_zalloc(num + 1);
   int i;
   va_list ap;
 
@@ -2226,19 +2224,6 @@ canon_deinit_printer(const stp_vars_t v, canon_init_t *init)
   canon_cmd(v,ESC40,0,0);
 }
 
-
-/*
- *  'alloc_buffer()' allocates buffer and fills it with 0
- */
-static unsigned char *
-canon_alloc_buffer(int size)
-{
-  unsigned char *buf= stp_malloc(size);
-  if (buf)
-    memset(buf,0,size);
-  return buf;
-}
-
 /*
  * 'advance_buffer()' - Move (num) lines of length (len) down one line
  *                      and sets first line to 0s
@@ -2333,6 +2318,7 @@ canon_print(const stp_printer_t printer,		/* I - Model */
   colormode_t colormode = canon_printhead_colors(ink_type,caps);
   const paper_t *pt;
   const canon_variable_inkset_t *inks;
+  stp_dither_data_t *dt;
 
   if (!stp_get_verified(nv))
     {
@@ -2498,7 +2484,7 @@ canon_print(const stp_printer_t printer,		/* I - Model */
   stp_deprintf(STP_DBG_CANON,"canon: buflength is %d!\n",buf_length);
 
   if (colormode==COLOR_MONOCHROME) {
-    black   = canon_alloc_buffer(buf_length*(delay_k+1));
+    black   = stp_zalloc(buf_length*(delay_k+1));
     cyan    = NULL;
     magenta = NULL;
     lcyan   = NULL;
@@ -2506,21 +2492,21 @@ canon_print(const stp_printer_t printer,		/* I - Model */
     yellow  = NULL;
     lyellow = NULL;
   } else {
-    cyan    = canon_alloc_buffer(buf_length*(delay_c+1));
-    magenta = canon_alloc_buffer(buf_length*(delay_m+1));
-    yellow  = canon_alloc_buffer(buf_length*(delay_y+1));
+    cyan    = stp_zalloc(buf_length*(delay_c+1));
+    magenta = stp_zalloc(buf_length*(delay_m+1));
+    yellow  = stp_zalloc(buf_length*(delay_y+1));
 
     if (colormode!=COLOR_CMY)
-      black = canon_alloc_buffer(buf_length*(delay_k+1));
+      black = stp_zalloc(buf_length*(delay_k+1));
     else
       black = NULL;
 
     if (colormode==COLOR_CCMMYK || colormode==COLOR_CCMMYYK) {
       use_6color= 1;
-      lcyan = canon_alloc_buffer(buf_length*(delay_lc+1));
-      lmagenta = canon_alloc_buffer(buf_length*(delay_lm+1));
+      lcyan = stp_zalloc(buf_length*(delay_lc+1));
+      lmagenta = stp_zalloc(buf_length*(delay_lm+1));
       if (colormode==CANON_INK_CcMmYyK)
-	lyellow = canon_alloc_buffer(buf_length*(delay_lc+1));
+	lyellow = stp_zalloc(buf_length*(delay_lc+1));
       else
 	lyellow = NULL;
     } else {
@@ -2581,7 +2567,6 @@ canon_print(const stp_printer_t printer,		/* I - Model */
     }
   stp_dither_set_black_lower(dither, k_lower);
   stp_dither_set_black_upper(dither, k_upper);
-  stp_dither_set_adaptive_limit(dither, .75);
 
   if ((inks = canon_inks(caps, res_code, colormode, bits))!=0)
     {
@@ -2620,8 +2605,8 @@ canon_print(const stp_printer_t printer,		/* I - Model */
     }
   stp_dither_set_density(dither, stp_get_density(nv));
 
-  in  = stp_malloc(image_width * image_bpp);
-  out = stp_malloc(image_width * out_bpp * 2);
+  in  = stp_zalloc(image_width * image_bpp);
+  out = stp_zalloc(image_width * out_bpp * 2);
 
   errdiv  = image_height / out_length;
   errmod  = image_height % out_length;
@@ -2659,6 +2644,13 @@ canon_print(const stp_printer_t printer,		/* I - Model */
     }
   }
 
+  dt = stp_create_dither_data();
+  stp_add_channel(dt, black, ECOLOR_K, 0);
+  stp_add_channel(dt, cyan, ECOLOR_C, 0);
+  stp_add_channel(dt, lcyan, ECOLOR_C, 1);
+  stp_add_channel(dt, magenta, ECOLOR_M, 0);
+  stp_add_channel(dt, lmagenta, ECOLOR_M, 1);
+  stp_add_channel(dt, yellow, ECOLOR_Y, 0);
 
   for (y = 0; y < out_length; y ++)
   {
@@ -2678,8 +2670,7 @@ canon_print(const stp_printer_t printer,		/* I - Model */
 		   have_sat_adjustment ? sat_adjustment : NULL);
     }
 
-    stp_dither(out, y, dither, cyan, lcyan, magenta, lmagenta,
-	       yellow, 0, black, duplicate_line, zero_mask);
+    stp_dither(out, y, dither, dt, duplicate_line, zero_mask);
 
     canon_write_line(nv, caps, ydpi,
 		     black,    delay_k,
@@ -2709,6 +2700,7 @@ canon_print(const stp_printer_t printer,		/* I - Model */
   }
   image->progress_conclude(image);
 
+  stp_free_dither_data(dt);
   stp_free_dither(dither);
 
   /*
@@ -2769,7 +2761,9 @@ const stp_printfuncs_t stp_canon_printfuncs =
   canon_print,
   canon_default_parameters,
   canon_describe_resolution,
-  stp_verify_printer_params
+  stp_verify_printer_params,
+  stp_start_job,
+  stp_end_job
 };
 
 /*
